##################################################
#*******vis_profile script************************
#converts DSAA .grd surfer files to PostScript****
#*author*:*Kirill Gadylshin***********************
#*mail*:*gadylshin@yahoo.com**********************
#*************************************************
##################################################

use strict;
use warnings;
use lib "../../COMMON/gmt";
use EnvConfig;
my $CONF_FILE_NAME = "gmt.conf";

my ($ar,$md,$iter) = ("","",);
my $M_PI = 3.1415926535;

#gmt PARAMS

my ($x_min,$x_max,$y_min,$y_max);
my ($cpt,$cpt_NaN);

my $ps_out;

my ($PAGE_WIDTH, $PAGE_HEIGHT);

my ($XMAP,$YMAP);
my ($TICK_X,$TICK_Y);

my ($CPT_MAIN,$Z0,$ZN,$N_STEPZ);
my ($CPT_ABS,$Z0_ABS,$ZN_ABS,$N_STEPZ_ABS);
my ($MARGIN_LEFT,$MARGIN_TOP);
my ($TITLE_FONT_SIZE,$TITLE_SHIFT_TOP);
my ($SCALE_THICKNESS,$SCALE_LENGTH,$SCALE_ORIENTATION,$SCALE_SHIFT);
my (@list_xyz);


sub usage()
{
    print "vis_hor.pl script takes no less then 2 arguments\n";   
    print "\$arg1  -  Area\n";
    print "\$arg2  -  Model\n";
    print "\$arg3  -  Iteration\n";
}

sub configurate()
{

    # Check
    print 'Checking gmt configuration... ';
    
    unless(-f $CONF_FILE_NAME)
    {
	print "FAILED : there is no configuration file\n" ;
	return(1);
    }
    print "OK\n";
    
    # Read
    print "Reading gmt configuration.. [$CONF_FILE_NAME] " ;

    my $res = get_configuration($CONF_FILE_NAME);
    
    unless(defined($res))
    {
	print "Undefined \$res";
	return(1);
    }

#    $LOGLEVEL = get_var("LOGLEVEL");
#    $LOGLEVEL ||= "ALL";
#    log_trace("LOGLEVEL=$LOGLEVEL");
#    set_log_level($LOGLEVEL);
    ($PAGE_WIDTH,$PAGE_HEIGHT) = (get_var("PAGE_WIDTH"),get_var("PAGE_HEIGHT"));
    
    ($XMAP,$YMAP) = (get_var("XMAP"),get_var("YMAP"));

    ($CPT_MAIN,$Z0,$ZN,$N_STEPZ) =(get_var("CPT_MAIN"),get_var("Z0"),get_var("ZN"),get_var("N_STEPZ"));
    ($CPT_ABS,$Z0_ABS,$ZN_ABS,$N_STEPZ_ABS) = (get_var("CPT_ABS"),get_var("Z0_ABS"),get_var("ZN_ABS"),get_var("N_STEPZ_ABS"));
    ($TICK_X,$TICK_Y) = (get_var("TICK_X"),get_var("TICK_Y"));
    ($MARGIN_LEFT,$MARGIN_TOP) = (get_var("MARGIN_LEFT"),get_var("MARGIN_TOP"));
    ($TITLE_FONT_SIZE,$TITLE_SHIFT_TOP) = (get_var("TITLE_FONT_SIZE"),get_var("TITLE_SHIFT_TOP"));
    ($SCALE_THICKNESS,$SCALE_LENGTH,$SCALE_ORIENTATION) = (get_var("SCALE_THICKNESS"),get_var("SCALE_LENGTH"),get_var("SCALE_ORIENTATION"));
    $SCALE_SHIFT = get_var("SCALE_SHIFT");
    $SCALE_ORIENTATION ||= "v";
    
    $CPT_MAIN = "../../COMMON/scales_cpt/$CPT_MAIN";
    $CPT_ABS = "../../COMMON/scales_cpt/$CPT_ABS";
    
    print "OK\n";

    return 0;
}


sub get_cpts($$$$)
{
    my ($cpt_main,$z0,$zn,$n_stepz) = @_;
    my ($cpt,$cpt_NaN);
    $cpt = "../../DATA/$ar/$md/data/dv_anom.cpt";
    $cpt_NaN = "../../DATA/$ar/$md/data/dv_anom_NaN.cpt";
    my $dz = ($zn-$z0+0.0)/$n_stepz;    

    my $str_cpt = "";
    
    my $cmd = "makecpt -C$cpt_main -T$z0/$zn/$dz > $cpt";
    system($cmd) == 0 or die "Error call makecpt : $?";
    
    open(CPT_IN,"<",$cpt) or die "Can't open $cpt : $!";
    open(CPT_OUT,">",$cpt_NaN) or die "Can't open $cpt_NaN : $!";
    
    my $line;
    my $zn_1 = $zn - 1;
    
    $line = <CPT_IN>;
    print CPT_OUT $line;
    $line = <CPT_IN>;
    print CPT_OUT $line;
    $line = <CPT_IN>;
    print CPT_OUT $line;
    print CPT_OUT "-10000 GRAY -500 GRAY\n";
    
    if (defined ($line = <CPT_IN>))
    {
        my ($zs,$rs,$gs,$bs,$zr,$rr,$gr,$br) = split(' ',$line);
        print CPT_OUT "-500 $rs $gs $bs $zs $rs $gs $bs\n";
	print CPT_OUT $line;
    }
    else
    {
        die "Error $cpt file format\n";
    }
    
    while (defined ($line = <CPT_IN>))
    {        
        if ($line =~ m/^B.*$/)
        {
            my ($zs,$rs,$gs,$bs,$zr,$rr,$gr,$br) = split(' ',$str_cpt);
            print CPT_OUT "$zn $rs $gs $bs 500 $rs $gs $bs\n";
        }
	print CPT_OUT $line;        
	$str_cpt = $line;
    }
    
    close CPT_IN;
    close CPT_OUT;
    
    $cmd = "makecpt -C$cpt_main -T$z0/$zn/$dz > $cpt";
    system($cmd) == 0 or die "Error call makecpt : $?";

    return ($cpt,$cpt_NaN);
}


sub main()
{
    if ($#ARGV + 1 < 3 )
    {
        usage();
        exit;
    }
    
    my ($grd_in,$xyz_out,$ztr_file);
    my ($title) = "";
    
    $ar = $ARGV[0];
    $md = $ARGV[1];
    $iter = $ARGV[2];
    
    $CONF_FILE_NAME = "../../DATA/$ar/gmt.conf";
    die "Configuration process has been FAILED\n" if (configurate() != 0 );
    $ENV{'PATH'} = $ENV{'PATH'} . ':/usr/lib/gmt/bin';    
    
    my $it = $iter;
    if ($iter < 10)
    {
	$iter = $iter + 0;
	$iter = " $iter";
    }
    $it = $it + 0;
    $grd_in = "../../FIG_files/result/dv_result" .$iter .".grd";
    $title = "Velocity anomalies, iteration ". $iter;        
    $ps_out = "../../PICS/ps/$ar/$md/anom" .$it .".ps";
    draw_profile($grd_in,$title,2);
    my $png_out = "../../PICS/png/$ar/$md/anom" .$it .".png";
    my $cmd = "convert -quality 100 -background white $ps_out $png_out";
    print $cmd . "\n";
    system($cmd) == 0 or print "Error call ps2png : $?";    


    $grd_in = "../../FIG_files/result/v_abs" .$iter .".grd";
    $title = "Absolute velocity, iteration ". $iter;    
    $ps_out = "../../PICS/ps/$ar/$md/v_abs" .$it .".ps";
    draw_profile($grd_in,$title,1);
    $png_out = "../../PICS/png/$ar/$md/v_abs" .$it .".png";
    $cmd = "convert -quality 100 -background white $ps_out $png_out";
    print $cmd . "\n";
    system($cmd) == 0 or print "Error call ps2png : $?";    
    
    #Draw ray paths
    my $bln_in = "../../FIG_files/rays/rays" . $iter . ".bln";
    my $bln_out = convertBln2Multisegm($bln_in);
    $title = "Ray paths after iteration $iter";
    $ps_out = "../../PICS/ps/$ar/$md/rays$it.ps";
    draw_ray_paths($bln_out,$title);
    $png_out = "../../PICS/png/$ar/$md/rays" .$it .".png";
    $cmd = "convert -quality 100 -background white $ps_out $png_out";
    print $cmd . "\n";
    system($cmd) == 0 or print "Error call ps2png : $?";    
    
    return 0;
}

sub draw_ray_paths($$)
{
    my ($bln_out,$title) = @_;
    
    ($cpt,$cpt_NaN) = get_cpts($CPT_ABS,$Z0_ABS,$ZN_ABS,$N_STEPZ_ABS);
    
  
    my $cmd;
    $cmd = "gmtset BASEMAP_TYPE PLAIN BASEMAP_FRAME_RGB black ANNOT_FONT_PRIMARY 0 ANNOT_FONT_SIZE_PRIMARY 10";
    $cmd = $cmd . " HEADER_FONT_SIZE 10 HEADER_OFFSET -0.5c PAGE_ORIENTATION portrait LABEL_FONT_SIZE 12";    
    system($cmd) == 0 or die "Error call gmtset : $?";
        
    $cmd="psxy -R$x_min/$x_max/$y_min/$y_max -JX${XMAP}c/${YMAP}c '$bln_out' ";
    $cmd = $cmd . " -B$TICK_X:\"Distance along profile\":/$TICK_Y:\"Depth\"::.\"$title\":WeSn";
    $cmd = $cmd . " -Wgray -M";
    my $OFF_X = $MARGIN_LEFT;
    my $OFF_Y = $PAGE_HEIGHT-$MARGIN_TOP-$YMAP;
    
    $cmd = $cmd . " -Xa${OFF_X}c -Ya${OFF_Y}c -K ";
    $cmd = $cmd . " --PAPER_MEDIA=Custom_${PAGE_WIDTH}cx${PAGE_HEIGHT}c > '$ps_out'";
    print "$cmd\n";
    system($cmd) == 0 or die "Error call psxy : $?";
 
    my $relief_bln = "../../DATA/$ar/inidata/relief.bln";
    $cmd="psxy -R$x_min/$x_max/$y_min/$y_max -JX${XMAP}c/${YMAP}c '$relief_bln' ";    
    $cmd = $cmd . " -W0.05c/black -H ";
    $cmd = $cmd . " -Xa${OFF_X}c -Ya${OFF_Y}c -O >> '$ps_out'";
    
    print "$cmd\n";
    system($cmd) == 0 or die "Error call psxy : $?";
    
}

sub draw_profile($$$)
{
    my ($grd_in,$title,$key_1abs_2anom) = @_;

    if ($key_1abs_2anom == 1)
    {
	($cpt,$cpt_NaN) = get_cpts($CPT_ABS,$Z0_ABS,$ZN_ABS,$N_STEPZ_ABS);
    }
    else
    {
	($cpt,$cpt_NaN) = get_cpts($CPT_MAIN,$Z0,$ZN,$N_STEPZ);
    }
    
    my $xyz_out = $grd_in . ".xyz";        
    convert2xyz($grd_in,$xyz_out);	
    if ($YMAP == 0)
    {
	$YMAP = $XMAP*($y_max-$y_min+0.0)/(($x_max-$x_min+0.0)*cos(($y_max+$y_min+0.0)/2.0*$M_PI/180.0));
    }
    
    my $cmd;
    $cmd = "gmtset BASEMAP_TYPE PLAIN BASEMAP_FRAME_RGB black ANNOT_FONT_PRIMARY 0 ANNOT_FONT_SIZE_PRIMARY 10";
    $cmd = $cmd . " HEADER_FONT_SIZE 10 HEADER_OFFSET -0.5c PAGE_ORIENTATION portrait LABEL_FONT_SIZE 12";    
    system($cmd) == 0 or die "Error call gmtset : $?";
    
    $cmd="pscontour -R$x_min/$x_max/$y_min/$y_max -JX${XMAP}c/${YMAP}c '$xyz_out' -C$cpt_NaN -I";
    $cmd = $cmd . " -B$TICK_X:\"Distance along profile\":/$TICK_Y:\"Depth\"::.\"$title\":WeSn";
    my $OFF_X = $MARGIN_LEFT;
    my $OFF_Y = $PAGE_HEIGHT-$MARGIN_TOP-$YMAP;
    
    $cmd = $cmd . " -Xa${OFF_X}c -Ya${OFF_Y}c -K ";
    $cmd = $cmd . " --PAPER_MEDIA=Custom_${PAGE_WIDTH}cx${PAGE_HEIGHT}c > '$ps_out'";
    print "$cmd\n";
    system($cmd) == 0 or die "Error call pscontour : $?";
   
   
    draw_scale($key_1abs_2anom); 
}


#draw plot map

sub draw_scale($)
{
    my $key_1abs_2anom = shift;
    return if ($SCALE_ORIENTATION eq "");
    
    my ($xpos,$ypos);
    if ($SCALE_ORIENTATION eq 'h')
    {
	$xpos = $MARGIN_LEFT + $XMAP*0.5;
	$ypos = $PAGE_HEIGHT-$MARGIN_TOP-$YMAP-$SCALE_SHIFT;
	my $cmd = "psscale -D${xpos}c/${ypos}c/${SCALE_LENGTH}c/${SCALE_THICKNESS}ch -S -C$cpt -O -K >> '$ps_out'";    
	system($cmd) == 0 or die "Error call psscale : $?";	
	$cmd = "echo \"0 0 12 0 1 BC velocity anomalies, %\"";
	if ($key_1abs_2anom == 1)
	{
	    $cmd = "echo \"0 0 12 0 1 BC P velocity, km/s\"";
	}
	$ypos = $ypos - $SCALE_THICKNESS - 1.3;
	$xpos = $xpos - $SCALE_LENGTH/2.0;
	$cmd = $cmd . " | pstext -R-10/10/-0.1/10 -JX${SCALE_LENGTH}c/10c -Xa${xpos} -Ya${ypos} -O >> '$ps_out'";
	system($cmd) == 0 or die "Error call pstext : $?";	
    }
    elsif ($SCALE_ORIENTATION eq 'v')
    {
	$xpos = $MARGIN_LEFT + $XMAP + $SCALE_SHIFT;
	$ypos = $PAGE_HEIGHT-$MARGIN_TOP-$YMAP*0.5;
	my $cmd = "psscale -D${xpos}c/${ypos}c/${SCALE_LENGTH}c/${SCALE_THICKNESS}c -S -C$cpt -O -K >> '$ps_out'";    
	system($cmd) == 0 or die "Error call psscale : $?";	
	$cmd = "echo \"0 0 12 90 1 TC velocity anomalies, %\"";
	if ($key_1abs_2anom == 1)
	{
	    $cmd = "echo \"0 0 12 90 1 TC P velocity, km/s\"";
	}	
	$xpos = $xpos + $SCALE_THICKNESS + 1.1;
	$ypos = $ypos - $SCALE_LENGTH/2.0;
	$cmd = $cmd . " | pstext -R0/1/-10/10 -JX10c/${SCALE_LENGTH}c -Xa${xpos} -Ya${ypos} -O >> '$ps_out'";
	system($cmd) == 0 or die "Error call pstext : $?";
    }
}

#convert DSAA surfer .grd file into xyz table
sub convert2xyz($$)
{
    my ($grd_in,$xyz_out) = @_;
    
    open(IN,"<",$grd_in) or die "Can't open '$grd_in' : $!";
    open(OUT,'>',$xyz_out) or die "Can't open $xyz_out : $!";
    
    print "Start converting $grd_in to xyz table...\n";
    
    my ($header,$nx,$ny,$z_min,$z_max,$line);
    

    die "Error: Incorrect file format" unless (defined ($header = <IN>) );
    
    die "Error: Incorrect file format" unless (defined ($line = <IN>) );
    ($nx,$ny) = split(' ',$line);

    die "Error: Incorrect file format" unless (defined ($line = <IN>) );
    ($x_min,$x_max) = split(' ',$line);
    
    die "Error: Incorrect file format" unless (defined ($line = <IN>) );
    ($y_min,$y_max) = split(' ',$line);
    
    die "Error: Incorrect file format" unless (defined ($line = <IN>) );
    ($z_min,$z_max) = split(' ',$line);
    
    die "Error: Incorrect file format" if ($ny < 2 || $nx < 2);
    

    my ($recn,$cnt) = ($nx*$ny,0);

    my ($dx,$dy) = (($x_max-$x_min+0.0)/($nx-1.0),($y_max-$y_min+0.0)/($ny-1.0));		
    
    while ($cnt < $recn && defined($line = <IN>))
    {      
 	for my $z (split(' ',$line)) 
	{
            $cnt++;
            my ($x,$y) = ($x_min+($cnt%$nx)*$dx,$y_min+($cnt/$nx)*$dy);
	    print OUT "$x $y $z\n";            
	}
    }

    die "Failed to convert. Not enough data in $grd_in. Terminated." if ( $cnt < $recn);
    warn "Too many data in $grd_in. Cutted." if (<IN>);
    
    
    close OUT;
    close IN;

    print "Converting OK. Processed $cnt grid nodes\n";
    
}


sub convertBln2Multisegm($)
{
    my $bln_in = shift;
    print "Convert '$bln_in' to multisegment gmt file...\n";
    my $bln_out = $bln_in .".dat";
    open(BLN_IN,"<",$bln_in) or die "Can't open '$bln_in' : $!";    
    open(BLN_OUT,'>',$bln_out) or die "Can't open $bln_out : $!";
    
    my $line;
    while (defined ($line = <BLN_IN>))
    {
	my $count = $line;
	$count = $count + 0;	
	print BLN_OUT ">\n";
	for my $i (1..$count)
	{
	    $line = <BLN_IN>;
	    print BLN_OUT $line;
	}
    }

    close(BLN_IN);
    close(BLN_OUT);
    
    print "Converting OK\n";
    return $bln_out;    
}

main();