##################################################
#*******vis_profile script************************
#converts DSAA .grd surfer files to PostScript****
#*author*:*Kirill Gadylshin***********************
#*mail*:*gadylshin@yahoo.com**********************
#*************************************************
##################################################

use strict;
use warnings;
use lib "../../COMMON/gmt";
use EnvConfig;
my $CONF_FILE_NAME = "gmt.conf";

my ($ar,$md,$iters) = ("","",);
my $M_PI = 3.1415926535;

#gmt PARAMS

my ($x_min,$x_max,$y_min,$y_max);
my ($cpt,$cpt_NaN);
my ($MAX_T,$MAX_X);

my $ps_out;

my ($PAGE_WIDTH, $PAGE_HEIGHT);

my ($XMAP,$YMAP);
my ($TICK_X,$TICK_Y);

my ($CPT_MAIN,$Z0,$ZN,$N_STEPZ);
my ($CPT_ABS,$Z0_ABS,$ZN_ABS,$N_STEPZ_ABS);
my ($MARGIN_LEFT,$MARGIN_TOP);
my ($TITLE_FONT_SIZE,$TITLE_SHIFT_TOP);
my ($SCALE_THICKNESS,$SCALE_LENGTH,$SCALE_ORIENTATION,$SCALE_SHIFT);
my (@list_xyz);


sub usage()
{
    print "vis_hor.pl script takes no less then 2 arguments\n";   
    print "\$arg1  -  Area\n";
    print "\$arg2  -  Model\n";
    print "\$arg3  -  Number of iterations\n";
    print "\$arg4  -  Max T\n";
    print "\$arg5  -  Max X\n";
}


sub configurate()
{

    # Check
    print 'Checking gmt configuration... ';
    
    unless(-f $CONF_FILE_NAME)
    {
	print "FAILED : there is no configuration file\n" ;
	return(1);
    }
    print "OK\n";
    
    # Read
    print "Reading gmt configuration.. [$CONF_FILE_NAME] " ;

    my $res = get_configuration($CONF_FILE_NAME);
    
    unless(defined($res))
    {
	print "Undefined \$res";
	return(1);
    }

#    $LOGLEVEL = get_var("LOGLEVEL");
#    $LOGLEVEL ||= "ALL";
#    log_trace("LOGLEVEL=$LOGLEVEL");
#    set_log_level($LOGLEVEL);
    ($PAGE_WIDTH,$PAGE_HEIGHT) = (get_var("PAGE_WIDTH"),get_var("PAGE_HEIGHT"));
    
    ($XMAP,$YMAP) = (get_var("XMAP"),get_var("YMAP"));

    ($CPT_MAIN,$Z0,$ZN,$N_STEPZ) =(get_var("CPT_MAIN"),get_var("Z0"),get_var("ZN"),get_var("N_STEPZ"));
    ($CPT_ABS,$Z0_ABS,$ZN_ABS,$N_STEPZ_ABS) = (get_var("CPT_ABS"),get_var("Z0_ABS"),get_var("ZN_ABS"),get_var("N_STEPZ_ABS"));
    ($TICK_X,$TICK_Y) = (get_var("TICK_X"),get_var("TICK_Y"));
    ($MARGIN_LEFT,$MARGIN_TOP) = (get_var("MARGIN_LEFT"),get_var("MARGIN_TOP"));
    ($TITLE_FONT_SIZE,$TITLE_SHIFT_TOP) = (get_var("TITLE_FONT_SIZE"),get_var("TITLE_SHIFT_TOP"));
    ($SCALE_THICKNESS,$SCALE_LENGTH,$SCALE_ORIENTATION) = (get_var("SCALE_THICKNESS"),get_var("SCALE_LENGTH"),get_var("SCALE_ORIENTATION"));
    $SCALE_SHIFT = get_var("SCALE_SHIFT");
    $SCALE_ORIENTATION ||= "v";
    
    $CPT_MAIN = "../../COMMON/scales_cpt/$CPT_MAIN";
    $CPT_ABS = "../../COMMON/scales_cpt/$CPT_ABS";
    
    print "OK\n";

    return 0;
}


sub get_cpts($$$$)
{
    my ($cpt_main,$z0,$zn,$n_stepz) = @_;
    my ($cpt,$cpt_NaN);
    $cpt = "../../DATA/$ar/$md/data/dv_anom.cpt";
    $cpt_NaN = "../../DATA/$ar/$md/data/dv_anom_NaN.cpt";
    my $dz = ($zn-$z0+0.0)/$n_stepz;    

    my $str_cpt = "";
    
    my $cmd = "makecpt -C$cpt_main -T$z0/$zn/$dz > $cpt";
    system($cmd) == 0 or die "Error call makecpt : $?";
    
    open(CPT_IN,"<",$cpt) or die "Can't open $cpt : $!";
    open(CPT_OUT,">",$cpt_NaN) or die "Can't open $cpt_NaN : $!";
    
    my $line;
    my $zn_1 = $zn - 1;
    
    $line = <CPT_IN>;
    print CPT_OUT $line;
    $line = <CPT_IN>;
    print CPT_OUT $line;
    $line = <CPT_IN>;
    print CPT_OUT $line;
    print CPT_OUT "-10000 GRAY -500 GRAY\n";
    
    if (defined ($line = <CPT_IN>))
    {
        my ($zs,$rs,$gs,$bs,$zr,$rr,$gr,$br) = split(' ',$line);
        print CPT_OUT "-500 $rs $gs $bs $zs $rs $gs $bs\n";
	print CPT_OUT $line;
    }
    else
    {
        die "Error $cpt file format\n";
    }
    
    while (defined ($line = <CPT_IN>))
    {        
        if ($line =~ m/^B.*$/)
        {
            my ($zs,$rs,$gs,$bs,$zr,$rr,$gr,$br) = split(' ',$str_cpt);
            print CPT_OUT "$zn $rs $gs $bs 500 $rs $gs $bs\n";
        }
	print CPT_OUT $line;        
	$str_cpt = $line;
    }
    
    close CPT_IN;
    close CPT_OUT;
    
    $cmd = "makecpt -C$cpt_main -T$z0/$zn/$dz > $cpt";
    system($cmd) == 0 or die "Error call makecpt : $?";

    return ($cpt,$cpt_NaN);
}


sub main()
{
    if ($#ARGV + 1 < 5 )
    {
        usage();
        exit;
    }
    
    my ($grd_in,$xyz_out,$ztr_file);
    my ($title) = "";
    
    $ar = $ARGV[0];
    $md = $ARGV[1];
    $iters = $ARGV[2];
    $MAX_T = $ARGV[3];
    $MAX_X = $ARGV[4];
    
    $MAX_X = $MAX_X + 0;
    $MAX_T = $MAX_T + 0.0;
    $CONF_FILE_NAME = "../../DATA/$ar/gmt.conf";
    die "Configuration process has been FAILED\n" if (configurate() != 0 );
    $ENV{'PATH'} = $ENV{'PATH'} . ':/usr/lib/gmt/bin';    
    
    $iters = $iters + 0;
    for my $i (1..$iters)
    {
	my $it;
	if ($i < 10 )
	{
	    $it = " $i";
	}
	else
	{
	    $it = $i;
	}
        my $t_obs_out = convertBln2Multisegm("../../FIG_files/stat/t_obs.bln");
        my $t_mod_out = convertBln2Multisegm("../../FIG_files/stat/t_mod${it}.bln");
        my $t_obs_dat = "../../FIG_files/stat/tobs.dat";
        my $t_mod_dat = "../../FIG_files/stat/tmod${it}.dat";
        $title = "Observed (red) and modeled (blue) times after iteration $i";
	$ps_out = "../../PICS/ps/$ar/$md/times${i}.ps";
        draw_times($t_obs_out,$t_mod_out,$t_obs_dat,$t_mod_dat,$title);
	my $png_out = "../../PICS/png/$ar/$md/times${i}.png";
	my $cmd = "convert -quality 500 -background white $ps_out $png_out";
	print $cmd . "\n";
	system($cmd) == 0 or print "Error call ps2png : $?";
    }
    
    
    return 0;
}

sub draw_times($$$$$)
{
    my $time_obs_out = shift;
    my $time_mod_out = shift;
    my $time_obs_dat = shift;
    my $time_mod_dat = shift;    
    my $title = shift;
    
    my $cmd;
    $cmd = "gmtset BASEMAP_TYPE PLAIN BASEMAP_FRAME_RGB black ANNOT_FONT_PRIMARY 0 ANNOT_FONT_SIZE_PRIMARY 10";
    $cmd = $cmd . " HEADER_FONT_SIZE 10 HEADER_OFFSET -0.5c PAGE_ORIENTATION portrait LABEL_FONT_SIZE 12";    
    system($cmd) == 0 or die "Error call gmtset : $?";
        
    $cmd="psxy -R0/$MAX_X/0/$MAX_T -JX${XMAP}c/${YMAP}c '$time_obs_out' ";
    $cmd = $cmd . " -B$TICK_X:\"Distance along profile\":/$TICK_Y:\"Time\"::.\"$title\":WeSn";
    $cmd = $cmd . " -Wred -M";
    my $OFF_X = $MARGIN_LEFT;
    my $OFF_Y = $PAGE_HEIGHT-$MARGIN_TOP-$YMAP;
    
    $cmd = $cmd . " -Xa${OFF_X}c -Ya${OFF_Y}c -K ";
    $cmd = $cmd . " --PAPER_MEDIA=Custom_${PAGE_WIDTH}cx${PAGE_HEIGHT}c > '$ps_out'";
    print "$cmd\n";
    system($cmd) == 0 or die "Error call psxy : $?";
    
    
    $cmd="psxy -R0/$MAX_X/0/$MAX_T -JX${XMAP}c/${YMAP}c '$time_mod_out' ";
    $cmd = $cmd . " -Xa${OFF_X}c -Ya${OFF_Y}c -O -K";
    $cmd = $cmd . " -Wblue -M >> '$ps_out'";
    print "$cmd\n";
    system($cmd) == 0 or die "Error call psxy : $?";

    $cmd="psxy -R0/$MAX_X/0/$MAX_T -JX${XMAP}c/${YMAP}c '$time_obs_dat' ";
    $cmd = $cmd . " -Xa${OFF_X}c -Ya${OFF_Y}c -O -K";
    $cmd = $cmd . " -Gred -Sc0.01c  >> '$ps_out'";
    print "$cmd\n";
    system($cmd) == 0 or die "Error call psxy : $?";
    
    $cmd="psxy -R0/$MAX_X/0/$MAX_T -JX${XMAP}c/${YMAP}c '$time_mod_dat' ";
    $cmd = $cmd . " -Xa${OFF_X}c -Ya${OFF_Y}c -O";
    $cmd = $cmd . " -Gblue -Sc0.01c  >> '$ps_out'";
    print "$cmd\n";
    system($cmd) == 0 or die "Error call psxy : $?";

}



#Convert *.bln file to Multisegment gmt format xy file
sub convertBln2Multisegm($)
{
    my $bln_in = shift;
    print "Convert '$bln_in' to multisegment gmt file...\n";
    my $bln_out = $bln_in .".dat";
    open(BLN_IN,"<",$bln_in) or die "Can't open '$bln_in' : $!";    
    open(BLN_OUT,'>',$bln_out) or die "Can't open $bln_out : $!";
    
    my $line;
    while (defined ($line = <BLN_IN>))
    {
	my $count = $line;
	$count = $count + 0;	
	print BLN_OUT ">\n";
	for my $i (1..$count)
	{
	    $line = <BLN_IN>;
	    print BLN_OUT $line;
	}
    }

    close(BLN_IN);
    close(BLN_OUT);
    
    print "Converting OK\n";
    return $bln_out;    
}

main();